import { Body, Controller, Post } from '@nestjs/common';
import { ApiBody, ApiOperation, ApiTags } from '@nestjs/swagger';
import { AuthService } from './auth.service';
import { AuthLoginAdminDto } from './dto/auth-login-admin.dto';
import { AuthLoginOperadorDto } from './dto/auth-login-operador.dto';
import { AuthLoginUsuarioDto } from './dto/auth-login-usuario.dto';

@Controller('auth')
@ApiTags('auth')
export class AuthController {
  constructor(private authService: AuthService) {}

  @Post('login-admin')
  @ApiOperation({
    description: 'Endpoint utilizado para el login del panel admin.',
  })
  @ApiBody({
    description: 'Todas las variables son obligatorias.',
    examples: {
      ejemplo: {
        value: {
          operador: '',
          password: '',
        },
      },
    },
  })
  loginAdmin(@Body() body: AuthLoginAdminDto) {
    return this.authService.loginAdmin(body.operador, body.password);
  }

  @Post('login-operador')
  @ApiOperation({
    description: 'Endpoint utilizado para el login del panel del operador.',
  })
  @ApiBody({
    description: 'Todas las variables son obligatorias.',
    examples: {
      ejemplo: {
        value: {
          operador: '',
          password: '',
          id_institucion: 200,
          id_modulo: 1,
        },
      },
    },
  })
  loginOperador(@Body() body: AuthLoginOperadorDto) {
    return this.authService.loginOperador(
      body.id_institucion,
      body.id_modulo,
      body.operador,
      body.password,
    );
  }

  @Post('login-usuario')
  @ApiOperation({ description: 'Endpoint utilizado para el login del usuario.' })
  @ApiBody({
    description: 'Todas las variables son obligatorias.',
    examples: { ejemplo: { value: { usuario: '', password: '' } } },
  })
  loginUsuario(@Body() body: AuthLoginUsuarioDto) {
    return this.authService.loginUsuario(body.usuario, body.password);
  }
}
