import { Injectable, NotFoundException } from '@nestjs/common';
import { InjectRepository } from '@nestjs/typeorm';
import { Repository } from 'typeorm';
import { Carrera } from '../institucion-carrera/entity/carrera.entity';
import { Institucion } from '../institucion/entity/institucion.entity';
import { InstitucionCarrera } from './entity/institucion-carrera.entity';
import { InstitucionService } from '../institucion/institucion.service';
import { Nivel } from './entity/nivel.entity';

@Injectable()
export class InstitucionCarreraService {
  constructor(
    @InjectRepository(InstitucionCarrera)
    private institucionCarreraRepository: Repository<InstitucionCarrera>,
    @InjectRepository(Carrera) private carreraRepository: Repository<Carrera>,
    @InjectRepository(Carrera) private nivelRepository: Repository<Nivel>,
    private institucionService: InstitucionService,
  ) {}

  findAllCarreras() {
    return this.carreraRepository.find();
  }

  findCarreraByIdCarrera(id_carrera: number) {
    return this.carreraRepository.findOne({ id_carrera }).then((carrera) => {
      if (!carrera) throw new NotFoundException('No existe esta carrera.');
      return carrera;
    });
  }

  findCarreraByCarrera(carrera: string, validarNoExiste = true) {
    return this.carreraRepository.findOne({ carrera }).then((carrera) => {
      if (validarNoExiste && !carrera)
        throw new NotFoundException('No existe esta carrera.');
      return carrera;
    });
  }

  findAllByIdInstitucion(id_institucion: number) {
    return this.institucionService
      .findById(id_institucion)
      .then((institucion) =>
        this.institucionCarreraRepository.find({ institucion }),
      );
  }

  findById(id_institucion_carrera: number) {
    return this.institucionCarreraRepository
      .findOne({ id_institucion_carrera })
      .then((institucionCarrera) => {
        if (!institucionCarrera)
          throw new NotFoundException('No existe esta institución carrera.');
        return institucionCarrera;
      });
  }

  async findByIdInstitucionIdCarrera(
    id_institucion: number | Institucion,
    id_carrera: number | Carrera,
  ) {
    const institucion =
      typeof id_institucion === 'number'
        ? await this.institucionService.findById(id_institucion)
        : id_institucion;
    const carrera =
      typeof id_carrera === 'number'
        ? await this.findCarreraByIdCarrera(id_carrera)
        : id_carrera;

    return this.institucionCarreraRepository.findOne({ carrera, institucion });
  }
}
