import {
  ConflictException,
  Injectable,
  NotFoundException,
} from '@nestjs/common';
import { InjectRepository } from '@nestjs/typeorm';
import { FindOperator, Like, Repository } from 'typeorm';
import { Institucion } from '../institucion/entity/institucion.entity';
import { InstitucionCarrera } from '../institucion-carrera/entity/institucion-carrera.entity';
import { TipoUsuario } from '../tipo-usuario/entity/tipo-usuario.entity';
import { Usuario } from './entity/usuario.entity';
import { BcryptService } from '../bcrypt/bcrypt.service';
import { InstitucionService } from '../institucion/institucion.service';
import { InstitucionCarreraService } from '../institucion-carrera/institucion-carrera.service';
import { NodemailerService } from '../nodemailer/nodemailer.service';
import { SoapClientService } from '../soap-client/soap-client.service';
import { TipoUsuarioService } from '../tipo-usuario/tipo-usuario.service';
import { Carrera } from 'src/institucion-carrera/entity/carrera.entity';

@Injectable()
export class UsuarioService {
  constructor(
    @InjectRepository(Usuario) private repository: Repository<Usuario>,
    private bcryptService: BcryptService,
    private institucionService: InstitucionService,
    private institucionCarreraService: InstitucionCarreraService,
    private nodemailerService: NodemailerService,
    private soapClientService: SoapClientService,
    private tipoUsuarioService: TipoUsuarioService,
  ) {}

  async create(
    usuario: string,
    nombre: string,
    institucion: Institucion,
    tipoUsuario: TipoUsuario,
    institucionCarrera?: InstitucionCarrera,
  ) {
    const data: {
      carrera?: InstitucionCarrera;
      institucion: Institucion;
      nombre: string;
      tipoUsuario: TipoUsuario;
      usuario: string;
    } = { institucion, nombre, tipoUsuario, usuario };

    if (institucionCarrera) data.carrera = institucionCarrera;
    return this.repository
      .save(this.repository.create(data))
      .then((usuario) => ({
        message: `Se creó correctamente un usuario.`,
        usuario,
      }));
  }

  correoPassword(password: string) {
    return `<h2>Estimado usuario, su registro se realizó con éxito.</h2>
<p>Para acceder al servicio debe hacerlo con los siguientes datos:</p>

<p>•	Usuario: Número de cuenta o número de trabajador</p>

<p>•	Contraseña: ${password}</p>

<p style=" text-align: justify; text-justify: inter-word;">
  El presente registro le permite hacer uso del préstamo del equipo de cómputo seleccionado, el cual puede ser utilizado únicamente al interior de la FES Acatlán.
  Para acceder al servicio es obligatorio y su responsabilidad leer los lineamientos "pcpuma-solicita" en la siguiente liga: <u>https://www.acatlan.unam.mx/pcpuma/Lineamientos.html</u>
  Hacemos de su conocimiento que la presente información es confidencial y debe ser tratada como tal, es de uso exclusivo para el usuario al que fue asignada y se encuentra protegida por la normatividad de la Universidad Nacional Autónoma de México en la materia, cualquier uso no autorizado o indebido se hará del conocimiento de la autoridad universitaria.
  Puede consultar nuestro aviso de privacidad en: <u>https://www.acatlan.unam.mx/normatividad</u>.
</p>

<p>Para mayor información o duda contáctenos por este medio.</p>

<p>Atte.</p>

<p>Pc Puma Solicita</p>`;
  }

  dgaeDgp(usuario: string) {
    // return this.findByUsuario(usuario).then(async (existeUsuario) => {
    //   if (!existeUsuario) {
    //     let esAlumno = false;

    //     await this.soapClientService
    //       .dgp(usuario)
    //       .then((res) => {
    //         if (!res) return this.soapClientService.dgp(usuario);
    //         esAlumno = true;
    //         return res;
    //       })
    //       .then((res) => {
    //         if (!res)
    //           throw new ConflictException('No es un número de cuenta válido.');

    //         // const data = {usuario,id_institucion:200,};
    //         // return this.create(usuario,, 5);
    //       });
    //   }
    //   if (existeUsuario.password)
    //     throw new ConflictException('Ya fue regisrado este usuario.');
    //   return existeUsuario;
    // });
    return { message: 'Pendiente' };
  }

  async findAll(filtros: {
    pagina: string;
    id_carrera?: string;
    id_institucion?: string;
    id_tipo_usuario?: string;
    nombre?: string;
    usuario?: string;
  }) {
    const busqueda: {
      institucionCarrera: { carrera?: Carrera; institucion?: Institucion };
      nombre?: FindOperator<string>;
      tipoUsuario?: TipoUsuario;
      usuario?: FindOperator<string>;
    } = { institucionCarrera: {} };
    const carrera = filtros.id_carrera
      ? await this.institucionCarreraService.findCarreraByIdCarrera(
          parseInt(filtros.id_carrera),
        )
      : null;
    const institucion = filtros.id_institucion
      ? await this.institucionService.findById(parseInt(filtros.id_institucion))
      : null;
    const tipoUsuario = filtros.id_tipo_usuario
      ? await this.tipoUsuarioService.findById(
          parseInt(filtros.id_tipo_usuario),
        )
      : null;

    if (filtros.usuario) busqueda.usuario = Like(`%${filtros.usuario}%`);
    if (filtros.nombre) busqueda.nombre = Like(`%${filtros.nombre}%`);
    if (carrera) busqueda.institucionCarrera.carrera = carrera;
    if (institucion) busqueda.institucionCarrera.institucion = institucion;
    if (tipoUsuario) busqueda.tipoUsuario = tipoUsuario;
    return this.repository.find({
      join: {
        alias: 'u',
        innerJoinAndSelect: {
          ic: 'u.institucionCarrera',
          c: 'ic.carrera',
          i: 'ic.institucion',
        },
      },
      where: busqueda,
      skip: (parseInt(filtros.pagina) - 1) * 25,
      take: 25,
      order: { usuario: 'ASC' },
    });
  }

  findById(id_usuario: number, validarNoExiste = true, password = false) {
    return this.repository.findOne({ id_usuario }).then((usuario) => {
      if (validarNoExiste && (!usuario || (password && !usuario.password)))
        throw new NotFoundException('No existe este usuario.');
      return usuario;
    });
  }

  findByUsuario(usuario: string, validarNoExiste = true, password = false) {
    return this.repository.findOne({ usuario }).then((usuario) => {
      if (validarNoExiste && (!usuario || (password && !usuario.password)))
        throw new NotFoundException('No existe este usuario.');
      return usuario;
    });
  }

  generarPassword = () => {
    const length = 8;
    const charset =
      'abcdefghijklmnopqrstuvwxyzABCDEFGHIJKLMNOPQRSTUVWXYZ0123456789';
    const n = charset.length;
    let password = '';

    for (let i = 0; i < length; i++)
      password += charset.charAt(Math.floor(Math.random() * n));
    return password;
  };

  passwordReset(id_usuario: number) {
    const password = this.generarPassword();

    return this.findById(id_usuario, true, true)
      .then((usuario) => {
        usuario.password = this.bcryptService.encriptar(password);
        return this.repository.save(usuario);
      })
      .then((usuario) =>
        this.nodemailerService.sendEmail({
          email: `${usuario.usuario}@pcpuma.acatlan.unam.mx`,
          subject: 'Credenciales Pc Puma',
          html: this.correoPassword(password),
        }),
      )
      .then((_) => ({
        message: 'Se reseteo correctamente la constraseña del usuario.',
      }));
  }

  registrar(id_usuario: number, telefono: string, correo?: string) {
    const password = this.generarPassword();

    return this.findById(id_usuario)
      .then((usuario) => {
        if (usuario.password)
          throw new ConflictException(
            'Ya fue registrado este número de cuenta.',
          );
        usuario.telefono = telefono;
        usuario.password = this.bcryptService.encriptar(password);
        return this.repository.save(usuario);
      })
      .then((usuario) =>
        this.nodemailerService.sendEmail({
          email: `${usuario.usuario}@pcpuma.acatlan.unam.mx`,
          subject: 'Credenciales Pc Puma',
          html: this.correoPassword(password),
        }),
      )
      .then((_) => ({ message: 'Se registro correctamente un usuario.' }));
  }

  update(attrs: Partial<Usuario>) {
    return this.findById(attrs.id_usuario, true, true)
      .then((usuario) => {
        Object.assign(usuario, attrs);
        return this.repository.save(usuario);
      })
      .then((_) => ({
        message: 'Se actualizo correctamente la información del usuario.',
      }));
  }
}
